from __future__ import absolute_import, unicode_literals
from collections import namedtuple
from datetime import datetime, timedelta
from workdays import workday


class DeadLine(namedtuple('Deadline', "step, days")):
    def limit(self, date=None):
        if not date:
            date = datetime.today()
        # cut microseconds
        date = date.replace(microsecond=0)
        if isinstance(self.days, Naturaldays):
            return date + timedelta(self.days)
        else:
            return workday(date, self.days)


class Workdays(int):
    def __new__(cls, *args, **kwargs):
        return super(Workdays, cls).__new__(cls, *args, **kwargs)


class Naturaldays(int):
    def __new__(cls, *args, **kwargs):
        return super(Naturaldays, cls).__new__(cls, *args, **kwargs)


class ProcessDeadline(object):

    steps = []

    @classmethod
    def get_deadline(cls, step, modifier=None):
        if modifier:
            steps = getattr(cls, 'steps_{0}'.format(modifier))
        else:
            steps = cls.steps
        for s in steps:
            if s.step == step:
                return s



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from .message_gas import MessageGas
from gestionatr.utils import get_rec_attr
from gestionatr.defs_gas import TIPUS_CONCEPTES, PEAJES_SEMPRE_CAPACIDAD, REFUND_RECTIFICATIVE_TYPES
from datetime import datetime, timedelta


class B7031(MessageGas):
    def __init__(self, xml, force_tipus=None, productes_exclosos=None):
        if productes_exclosos is None:
            productes_exclosos = []
        super(MessageGas, self).__init__(xml, force_tipus=force_tipus)
        self.productes_exclosos = productes_exclosos

    @property
    def datosempresaemisora(self):
        tree = 'datosempresaemisora'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return Datosempresaemisora(data)
        else:
            return False

    @property
    def datosempresadestino(self):
        tree = 'datosempresadestino'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return Datosempresadestino(data)
        else:
            return False

    @property
    def facturas(self):
        data = []
        for d in get_rec_attr(self.obj, 'factura', []):
            data.append(Factura(d, self.productes_exclosos))
        return data

    def get_datosempresaemisora(self):
        tree = 'datosempresaemisora'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            aux = Datosempresaemisora(data)
            return {
                'numdocumento': aux.numdocumento,
                'razonsocial': aux.razonsocial,
                'direccion': aux.direccion,
                'municipio': aux.municipio,
                'regmercantil': aux.regmercantil,
            }
        else:
            return False

    def get_datosempresadestino(self):
        tree = 'datosempresaemisora'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            aux = Datosempresadestino(data)
            return {
                'numdocumento': aux.numdocumento,
                'razonsocial': aux.razonsocial,
                'direccion': aux.direccion,
                'municipio': aux.municipio,
            }
        else:
            return False

    def get_facturas_atr(self):
        return [f for f in self.facturas if not f.is_only_conceptes()]

    def get_facturas_otros(self):
        if self.codproceso == '33':
            return []
        return [f for f in self.facturas if f.is_only_conceptes()]


class Datosempresadestino(object):
    def __init__(self, data):
        self.obj = data

    @property
    def numdocumento(self):
        tree = 'numdocumento'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def razonsocial(self):
        tree = 'razonsocial'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def direccion(self):
        tree = 'direccion'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def municipio(self):
        tree = 'municipio'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False


class Datosempresaemisora(Datosempresadestino):

    @property
    def regmercantil(self):
        tree = 'regmercantil'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False


class Factura(object):
    def __init__(self, data, productes_exclosos):
        self.obj = data
        self.productes_exclosos = productes_exclosos

    @property
    def rangopresiondiseno(self):
        tree = 'rangopresiondiseno'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def telemedida(self):
        tree = 'telemedida'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def tipogasinera(self):
        tree = 'tipogasinera'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def metodofacturacion(self):
        tree = 'metodofacturacion'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            # Si no ens el diuen, el calcularem! En base a la tarifa i a la telemedida ho podem saber
            tarifa = self.tipopeaje
            telemedida = self.telemedida
            if not tarifa or not telemedida:  # No hauria de passar ja que son camps obligatoris en les noves tarifes...
                return False
            elif tarifa in PEAJES_SEMPRE_CAPACIDAD:  # A partir de la 7 sempre son capacidad
                return '1'
            elif telemedida == 'N':  # Les mes baixes, si no tenen telemedida segur que son per client
                return '2'
            # Si arribem a aquest punt vol dir que el F1 esta malament ja que si es una tarifa de les baixes i
            # te telemedida, ens han de concretar com es factura sempre
            return False

    @property
    def cups(self):
        tree = 'cups'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def contratosimultaneo(self):
        tree = 'contratosimultaneo'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def codigoproducto(self):
        tree = 'codigoproducto'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def tipoproducto(self):
        tree = 'tipoproducto'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def arrastrapenalizacion(self):
        tree = 'arrastrapenalizacion'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def provincia(self):
        tree = 'provincia'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def municipio(self):
        tree = 'municipio'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def codpostal(self):
        tree = 'codpostal'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def tipovia(self):
        tree = 'tipovia'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def descalle(self):
        tree = 'descalle'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def numfinca(self):
        tree = 'numfinca'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def portal(self):
        tree = 'portal'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def escalera(self):
        tree = 'escalera'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def piso(self):
        tree = 'piso'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def puerta(self):
        tree = 'puerta'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def municipio_red(self):
        tree = 'municipioRed'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def tipodocumento(self):
        tree = 'tipodocumento'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def numdocumento(self):
        tree = 'numdocumento'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def nombre(self):
        tree = 'nombre'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def apellido1(self):
        tree = 'apellido1'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def apellido2(self):
        tree = 'apellido2'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def tipofactura(self):
        tree = 'tipofactura'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def clasefact(self):
        tree = 'clasefact'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def numfactorigen(self):
        tree = 'numfactorigen'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def fecfactura(self):
        tree = 'fecfactura'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def numfactura(self):
        tree = 'numfactura'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    def get_origin(self):
        return self.numfactura

    @property
    def tipofacturacion(self):
        tree = 'tipofacturacion'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def tipopeaje(self):
        tree = 'tipopeaje'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def feccontable(self):
        tree = 'feccontable'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def fecpago(self):
        tree = 'fecpago'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def importetotal(self):
        tree = 'importetotal'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return float(data.text)
        else:
            return False

    @property
    def saldo_total_a_cobrar(self):
        tree = 'SaldoTotalACobrar'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return float(data.text)
        else:
            return False

    @property
    def idremesa(self):
        tree = 'idremesa'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def tipopenalizacion(self):
        tree = 'tipopenalizacion'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def observaciones1(self):
        tree = 'observaciones1'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def observaciones2(self):
        tree = 'observaciones2'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def telefurgencias(self):
        tree = 'telefurgencias'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def listaboe(self):
        data = []
        obj = self.obj
        if (hasattr(obj, 'listaboe') and
                hasattr(obj.listaboe, 'boe')):
            for d in obj.listaboe.boe:
                data.append(Boe(d))
        return data

    @property
    def listaconceptos(self):
        data = []
        obj = self.obj
        if (hasattr(obj, 'listaconceptos') and
                hasattr(obj.listaconceptos, 'concepto')):
            for d in obj.listaconceptos.concepto:
                data.append(Concepto(d))
        return data

    def get_coeficientecortoplazo(self):
        for d in self.listaconceptos:
            if d.coeficientecortoplazo and float(d.coeficientecortoplazo):
                return float(d.coeficientecortoplazo)
        return 0.0

    @property
    def listamedidores(self):
        data = []
        obj = self.obj
        if (hasattr(obj, 'listamedidores') and
                hasattr(obj.listamedidores, 'medidor')):
            for d in obj.listamedidores.medidor:
                data.append(Medidor(d))
        return data

    @property
    def listafacturasinspeccion(self):
        data = []
        obj = self.obj
        if (hasattr(obj, 'listafacturasinspeccion') and
                hasattr(obj.listafacturasinspeccion, 'facturainspeccion')):
            for d in obj.listafacturasinspeccion.facturainspeccion:
                data.append(Facturasinspeccion(d))
        return data

    @property
    def lista_contactos(self):
        data = []
        obj = self.obj
        if (hasattr(obj, 'listaContactos') and
                hasattr(obj.listaContactos, 'contacto')):
            for d in obj.listaContactos.contacto:
                data.append(Contacto(d))
        return data

    @property
    def historial_consumos(self):
        data = []
        obj = self.obj
        if (hasattr(obj, 'historialConsumos') and
                hasattr(obj.historialConsumos, 'consumo')):
            for d in obj.historialConsumos.consumo:
                data.append(Consumo(d))
        return data

    @property
    def imputacion_costes(self):
        tree = 'imputacionCostes'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return Imputacioncostes(data)
        else:
            return False

    @property
    def mediaconsumo(self):
        tree = 'mediaconsumo'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return MediaConsumo(data)
        else:
            return False

    @property
    def codtbai(self):
        tree = 'codtbai'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    def get_periode_factura(self):
        """Retorna tupla amb (data inici,  data fi) de la factura:
            - data inici: la mes antiga de les fecdesde dels conceptes
            - data fi: la mes nova de les fechasta dels conceptes
        """
        return (
            min([x.fecdesde for x in self.listaconceptos if x.codconcepto not in self.productes_exclosos]),
            max([x.fechasta for x in self.listaconceptos if x.codconcepto not in self.productes_exclosos])
        )

    def get_periode_factura_peatges(self):
        """Retorna tupla amb (data inici,  data fi) de la factura:
            - data inici: la mes antiga de les fecdesde dels conceptes
            - data fi: la mes nova de les fechasta dels conceptes
        """
        return (
            min([x.fecdesde for x in self.listaconceptos if
                 "tvariable" in TIPUS_CONCEPTES.get(x.codconcepto, "") or "tfix" in TIPUS_CONCEPTES.get(x.codconcepto, "")]
                ),
            max([x.fechasta for x in self.listaconceptos if
                 "tvariable" in TIPUS_CONCEPTES.get(x.codconcepto, "") or "tfix" in TIPUS_CONCEPTES.get(x.codconcepto, "")]
                )
        )

    def is_only_conceptes(self):
        has_only_conceptes = True
        for type in self.get_linies_factura_by_type():
            if type not in ['informacio', 'altres']:
                has_only_conceptes = False
        return has_only_conceptes

    def get_create_invoice_params(self):
        return {
            'tipo_rectificadora': self.clasefact or self.indfacturarect or 'N',
            'date_invoice': self.fecfactura,
            'check_total': -1 * self.importetotal if self.clasefact in REFUND_RECTIFICATIVE_TYPES else self.importetotal,
            'origin': self.get_origin(),
            'origin_date_invoice': self.fecfactura,
            'reference': self.get_origin(),
            'tipo_factura': self.tipofactura
        }

    def get_linies_factura_by_type(self):
        res = {}
        res_to_join = {}
        to_join = {
            'tfixe_cargos': 'tfixe',
            'tfixe_altres': 'tfixe',
            'tvariable_altres': 'tvariable',
            'tfixe_demandad_altres': 'tfixe_demandad',
        }
        for concepte in self.listaconceptos:
            tipus = TIPUS_CONCEPTES.get(concepte.codconcepto, "altres")
            if tipus == "impost":
                continue
            if tipus in to_join.keys():
                aux_res = res_to_join
            else:
                aux_res = res
            aux_res.setdefault(tipus, {'total': 0.0, 'lines': []})
            aux_res[tipus]['lines'] += [concepte]
            new_total = aux_res[tipus]['total'] + concepte.importe
            aux_res[tipus]['total'] = round(new_total, 2)

        for tipus, info in res_to_join.items():
            tipus_join = to_join.get(tipus)
            if not res.get(tipus_join):
                res[tipus_join].setdefault(tipus, {'total': 0.0, 'lines': []})
            base = 1.0
            res[tipus_join]['total'] += info['total']
            for l in info['lines']:
                for l2 in res[tipus_join]['lines']:
                    if l2.unidad == l.unidad:
                        l2.precunidad += round(l.precunidad * base, 9)
        return res

    def get_comptadors(self):
        """Retorna totes les lectures en una llista de comptadors"""
        comptadors_agrupats = {}
        for medidor in self.listamedidores:
            comptadors_agrupats.setdefault(
                medidor.numseriemedidor, []
            ).append(medidor)

        comptadors = []
        for llista_aparells in comptadors_agrupats.values():
            aparell_multi = MultiModeloAparato(llista_aparells)

            di, df = aparell_multi.get_dates_inici_i_final()
            comptadors.append((di, df, aparell_multi))
        return [a[2] for a in sorted(comptadors, lambda x,y: cmp(x[0], y[0]))]

    def get_info_lloguer(self):
        lloguers = []
        total = 0
        info = self.get_linies_factura_by_type().get('lloguer', False)
        if info:
            lloguers = info['lines']
            total = info['total']
        return lloguers, total

    def get_remesa(self):
        return {
            'id_remesa': self.idremesa,
            'fecha_valor_remesa': self.fecpago
        }

class Boe(object):
    def __init__(self, data):
        self.obj = data

    @property
    def numboe(self):
        tree = 'numboe'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def fecboe(self):
        tree = 'fecboe'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False


class Concepto(object):
    def __init__(self, data):
        self.obj = data
        self._precunidad = 0.0

    @property
    def coeficientecortoplazo(self):
        tree = 'coeficientecortoplazo'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def multexcesocaudal(self):
        tree = 'multexcesocaudal'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def diascapacidadcontratada(self):
        tree = 'diascapacidadcontratada'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def horascapacidadcontratada(self):
        tree = 'horascapacidadcontratada'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def fecdesde(self):
        tree = 'fecdesde'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def fechasta(self):
        tree = 'fechasta'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def unidad(self):
        tree = 'unidad'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return float(data.text)
        else:
            return False

    @property
    def precunidad(self):
        if self._precunidad:
            return self._precunidad
        tree = 'precunidad'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return float(data.text)
        else:
            return False

    @precunidad.setter
    def precunidad(self, value):
        self._precunidad = value

    @property
    def importe(self):
        tree = 'importe'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return float(data.text)
        else:
            return False

    @property
    def codconcepto(self):
        tree = 'codconcepto'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def desconcepto(self):
        tree = 'desconcepto'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def porcentajeconcepto(self):
        tree = 'porcentajeconcepto'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return float(data.text)
        else:
            return False

    @property
    def impuestoconcepto(self):
        tree = 'impuestoconcepto'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def codtipoimpuesto(self):
        tree = 'codtipoimpuesto'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def porcentajeimpcto(self):
        tree = 'porcentajeimpcto'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return float(data.text)
        else:
            return False

    @property
    def umconcepto(self):
        tree = 'umconcepto'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def aparatoconcepto(self):
        tree = 'aparatoconcepto'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def observaciones(self):
        tree = 'observaciones'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def fec_desde_prorrateo(self):
        tree = 'fecDesdeProrrateo'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def tipo_interes_demora(self):
        tree = 'tipoInteresDemora'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False


class Medidor(object):
    def __init__(self, data):
        self.obj = data

    @property
    def excesocaudal(self):
        tree = 'excesocaudal'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def um(self):
        tree = 'um'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def feclecant(self):
        tree = 'feclecant'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def horalecant(self):
        tree = 'horalecant'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def feclecact(self):
        tree = 'feclecact'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def horalecact(self):
        tree = 'horalecact'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def serializada(self):
        tree = 'serializada'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def restadeserializada(self):
        tree = 'restadeserializada'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def cupsresta(self):
        tree = 'cupsresta'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def aparato(self):
        tree = 'aparato'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def medicion(self):
        tree = 'medicion'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def modelomedidor(self):
        tree = 'modelomedidor'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def numseriemedidor(self):
        tree = 'numseriemedidor'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def unipres(self):
        tree = 'unipres'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def presatm(self):
        tree = 'presatm'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def presionsuministro(self):
        tree = 'presionsuministro'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def temp(self):
        tree = 'temp'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def factorconver(self):
        tree = 'factorconver'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return float(data.text)
        else:
            return False

    @property
    def factork(self):
        tree = 'factork'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return float(data.text)
        else:
            return False

    @property
    def pcs(self):
        tree = 'pcs'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return float(data.text)
        else:
            return False

    @property
    def zeta(self):
        tree = 'zeta'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def densidad(self):
        tree = 'densidad'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def n2(self):
        tree = 'n2'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def co2(self):
        tree = 'co2'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def h2(self):
        tree = 'h2'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def consumokwh(self):
        tree = 'consumokwh'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return float(data.text)
        else:
            return False

    @property
    def consumoereal(self):
        tree = 'consumoereal'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return float(data.text)
        else:
            return False

    @property
    def consumoreg(self):
        tree = 'consumoreg'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return float(data.text)
        else:
            return False

    @property
    def codajuste(self):
        tree = 'codajuste'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def ajuste(self):
        tree = 'ajuste'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return float(data.text)
        else:
            return False

    @property
    def qdaplicado(self):
        tree = 'qdaplicado'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return float(data.text)
        else:
            return False

    @property
    def qdmaximo(self):
        tree = 'qdmaximo'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return float(data.text)
        else:
            return False

    @property
    def fecqdmax(self):
        tree = 'fecqdmax'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def dqmedio(self):
        tree = 'dqmedio'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def qdcontratado(self):
        tree = 'qdcontratado'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return float(data.text)
        else:
            return False

    @property
    def motivolec(self):
        tree = 'motivolec'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def tipo_dh(self):
        tree = 'tipoDH'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def perlec(self):
        tree = 'perlec'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def capacidadcontador(self):
        tree = 'capacidadcontador'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def observaciones1(self):
        tree = 'observaciones1'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def observaciones2(self):
        tree = 'observaciones2'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def listanumeradores(self):
        data = []
        obj = self.obj
        if (hasattr(obj, 'listanumeradores') and
                hasattr(obj.listanumeradores, 'numerador')):
            for d in obj.listanumeradores.numerador:
                data.append(Numerador(d))
        return data

    def get_lectures_info(self):
        res = []
        if not hasattr(self, 'meters'):
            self.meters = [self]
        factor_ultima_lectura = False
        for meter in self.meters:
            fecha_desde = meter.feclecant
            fecha_actual = meter.feclecact
            if fecha_desde == fecha_actual and meter.horalecact in ("23:00:00", "24:00:00"):
                # A vegades envien una lectura a les 00 i l'altre a les 23 i son del mateix dia.
                # Es equivalent a enviar una a les 23 del dia anterior i l'altre a les 23 del dia actual. per tant,
                # restem un dia a la lectura desde
                fecha_desde = (datetime.strptime(fecha_desde, "%Y-%m-%d") - timedelta(days=1)).strftime("%Y-%m-%d")
            comptador = meter.numseriemedidor
            periode = meter.tipo_dh
            ajust = float(meter.ajuste)
            motiu = meter.codajuste
            factor_k = float(meter.factork)
            pcs = float(meter.pcs)
            pressio_subministrament = float(meter.presionsuministro)
            qdaplicado = float(meter.qdaplicado)
            qdmaximo = float(meter.qdmaximo)
            fecqdmax = meter.fecqdmax
            dqmedio = float(meter.dqmedio)
            qdcontratado = float(meter.qdcontratado)
            excesocaudal = float(meter.excesocaudal)
            temperatura_gas = float(meter.temp)
            pressio_atmosferica = float(meter.presatm)
            consum_kwh_facturat = float(meter.consumokwh)
            for numerador in meter.listanumeradores:
                lectura_desde_m3 = float(numerador.lectant)
                lectura_actual_m3 = float(numerador.lecact)
                origen_desde = numerador.tipolec
                origen_actual = numerador.tipolec
                tipo_lect_num = numerador.tipolecnum
                lectura_desde = lectura_desde_m3 * float(meter.factorconver)
                lectura_actual = lectura_actual_m3 * float(meter.factorconver)
                vals = {
                    'pressio_subministrament': pressio_subministrament,
                    'lectura_desde': lectura_desde,
                    'lectura_actual': lectura_actual,
                    'lectura_desde_m3': lectura_desde_m3,
                    'lectura_actual_m3': lectura_actual_m3,
                    'fecha_desde': fecha_desde,
                    'fecha_actual': fecha_actual,
                    'comptador': comptador,
                    'origen_desde': origen_desde,
                    'origen_actual': origen_actual,
                    'periode': periode or "P1",
                    'ajust': ajust,
                    'motiu': motiu,
                    'tipo_lect_num': tipo_lect_num,
                    'factor_k': factor_k,
                    'pcs': pcs,
                    'ometre': numerador.aparatorelevante == 'N',
                    'consum_m3': float(numerador.consumo),
                    'consum': float(numerador.consumo) * float(meter.factorconver),
                    'qdaplicado': float(qdaplicado),
                    'qdmaximo': float(qdmaximo),
                    'fecqdmax': fecqdmax,
                    'dqmedio': float(dqmedio),
                    'qdcontratado': float(qdcontratado),
                    'excesocaudal': float(excesocaudal),
                    'temperatura_gas': float(temperatura_gas),
                    'pressio_atmosferica': float(pressio_atmosferica),
                }

                # Si el consum que calculem amb el factor_k i el pcs no dona el consum calculat amb el factor de conversio
                # pero si fem servir el factor de conversio informat per la distri si que dona, modfiquem el factor_k perque
                # el factor de conversio calculat sigui igual al informat i aixi el consum també quadri.
                # TODO
                # Revisant un cas concret, sembla que realment ens falta aplicar la zeta, per aixo no quadra.
                # Normalment es 1 pero pot no ser-ho. Aixó es una mandanga per no ahver de treballar amb la zeta.
                consum_calculat_sefons_fomrula = round(vals['consum_m3'] * vals['pcs'] * vals['factor_k'], 3)
                consum_calculat_segons_factor = round(vals['consum'], 3)
                if consum_calculat_segons_factor != consum_calculat_sefons_fomrula and vals['pcs']:
                    consum_calculat = round(vals['consum_m3'] * meter.factorconver, 3)
                    if consum_calculat_segons_factor == consum_calculat:
                        vals['factor_k'] = meter.factorconver / vals['pcs']

                # Unaltre cas concret de ***
                # Hi ha distris que tot i tindre lectures diaries no apliquen el factor de conversio registart en
                # cada dia, sino que paliquen a totes les lectures de tots els dies el factor de conversio de la
                # ultima lectura
                # Per tant: si el consum facturat per la distri no coincideix amb el consum calculat, ero aplicant
                # el faactor de conversio del ultim dia siq ue aplica, recalculem el factor k perque quadri tot.
                if consum_kwh_facturat != consum_calculat_segons_factor and vals['pcs'] and vals['consum_m3']:
                    if not factor_ultima_lectura and self.meters[-1].factorconver:
                        factor_ultima_lectura = float(self.meters[-1].factorconver)
                    if factor_ultima_lectura:
                        vals['factor_k'] = consum_kwh_facturat / (vals['consum_m3'] * vals['pcs'])
                        vals['consum'] = consum_kwh_facturat
                res.append(vals)
        return res

    def get_giro(self):
        return max([10**int(l.digmed) for l in self.listanumeradores])


class Numerador(object):
    def __init__(self, data):
        self.obj = data

    @property
    def num(self):
        tree = 'num'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def digmed(self):
        tree = 'digmed'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def digdecmed(self):
        tree = 'digdecmed'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def factmulmed(self):
        tree = 'factmulmed'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def lectant(self):
        tree = 'lectant'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def lecact(self):
        tree = 'lecact'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def tipolec(self):
        tree = 'tipolec'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def consumo(self):
        tree = 'consumo'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def tipolecnum(self):
        tree = 'tipolecnum'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def aparatorelevante(self):
        tree = 'aparatorelevante'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def observaciones(self):
        tree = 'observaciones'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False


class Facturasinspeccion(object):
    def __init__(self, data):
        self.obj = data

    @property
    def numdocumentoinstalador(self):
        tree = 'numdocumentoinstalador'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def razonsocialinstalador(self):
        tree = 'razonsocialinstalador'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def numfacturainspeccion(self):
        tree = 'numfacturainspeccion'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False


class Contacto(object):
    def __init__(self, data):
        self.obj = data

    @property
    def denominacion(self):
        tree = 'denominacion'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def url(self):
        tree = 'url'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def email(self):
        tree = 'email'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def telefono(self):
        tree = 'telefono'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False


class Consumo(object):
    def __init__(self, data):
        self.obj = data

    @property
    def fecinicioperiodo(self):
        tree = 'fecinicioperiodo'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def fecfinperiodo(self):
        tree = 'fecfinperiodo'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def consumoperiodo(self):
        tree = 'consumoperiodo'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False


class Imputacioncostes(object):
    def __init__(self, data):
        self.obj = data

    @property
    def pcttasacnmc(self):
        tree = 'pcttasacnmc'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def pctcuotagts(self):
        tree = 'pctcuotagts'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False


class MediaConsumo(object):
    def __init__(self, data):
        self.obj = data

    @property
    def mediaconsperiodofact5A(self):
        tree = 'mediaconsperiodofact5A'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def mediaconsperiodofact(self):
        tree = 'mediaconsperiodofact'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False


# Datos para B7032
class B7032(B7031):
    pass


class B7033(B7032):
    pass


class Factura(Factura):

    @property
    def numpseudofactura(self):
        tree = 'numpseudofactura'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    def get_origin(self):
        res = super(Factura, self).get_origin()
        if self.numpseudofactura:
            return (res or "") + " - " + (self.numpseudofactura or "")
        else:
            return res

    @property
    def numfacturarect(self):
        tree = 'numfacturarect'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def numfactorigen(self):
        if self.numfacturarect:
            return self.numfacturarect
        else:
            return super(Factura, self).numfactorigen

    @property
    def indfacturarect(self):
        tree = 'indfacturarect'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def clasefact(self):
        if self.indfacturarect:
            return self.indfacturarect
        else:
            return super(Factura, self).clasefact


# Let the user say only B70
class B70(B7031):
    pass


class MultiModeloAparato(Medidor):

    def __init__(self, meter_list):
        self.meters = meter_list
        super(MultiModeloAparato, self).__init__(meter_list[0])

    def _get_single_attribute(self, attribute):
        for meter in self.meters:
            attr_val = getattr(meter, attribute, None)
            if attr_val is not None:
                return attr_val

        return None

    def _get_list_attribute(self, attribute):
        res = []

        for meter in self.meters:
            if hasattr(meter, attribute):
                res += getattr(meter, attribute)

        return res

    def get_dates_inici_i_final(self):
        data_inici = ''
        data_final = ''
        for lect in self.meters:
            data_in_compt = datetime.strptime(
                lect.feclecant, '%Y-%m-%d'
            )
            data_fi_compt = datetime.strptime(
                lect.feclecact, '%Y-%m-%d'
            )

            if not data_inici or data_in_compt < data_inici:
                data_inici = data_in_compt
            if not data_final or data_in_compt > data_final:
                data_final = data_fi_compt

        return data_inici, data_final

    @property
    def feclecant(self):
        return self._get_single_attribute('feclecant')

    @property
    def feclecact(self):
        return self._get_single_attribute('feclecact')

    @property
    def modelomedidor(self):
        return self._get_single_attribute('modelomedidor')

    @property
    def numseriemedidor(self):
        return self._get_single_attribute('numseriemedidor')

    @property
    def factorconver(self):
        return self._get_single_attribute('factorconver')

    @property
    def factork(self):
        return self._get_single_attribute('factork')

    @property
    def pcs(self):
        return self._get_single_attribute('pcs')

    @property
    def consumokwh(self):
        return self._get_single_attribute('consumokwh')

    @property
    def codajuste(self):
        return self._get_single_attribute('codajuste')

    @property
    def ajuste(self):
        return self._get_single_attribute('ajuste')

    @property
    def motivolec(self):
        return self._get_single_attribute('motivolec')

    @property
    def tipo_dh(self):
        return self._get_single_attribute('tipo_dh')

    @property
    def listanumeradores(self):
        return self._get_list_attribute('listanumeradores')
    @property
    def presionsuministro(self):
        return self._get_single_attribute('presionsuministro')


def agrupar_lectures_per_data(lectures):
    """Retorna un diccionari de llistes en què les
       claus són les dates inicial i final de les lectures
    """
    lect = {}
    for i in lectures:
        key = (i['fecha_desde'], i['fecha_actual'])
        if key not in lect:
            lect[key] = []
        lect[key].append(i)
    return lect


def obtenir_data_inici_i_final(dic):
    """Retorna la data inicial i final del diccionari retornat
       per la funció agrupar_lectures_per_data()
    """
    inici_conjunt = None
    final_conjunt = None
    for keys in dic.keys():
        if not inici_conjunt or inici_conjunt > keys[0]:
            inici_conjunt = keys[0]
        if not final_conjunt or final_conjunt < keys[1]:
            final_conjunt = keys[1]

    return inici_conjunt, final_conjunt



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from libcomxml.core import XmlModel, XmlField
from gestionatr.output.messages.base_gas import Heading

from gestionatr.output.messages.base import rep_decimal

from .sw_a1_41 import Registerdoc, Registerdoclist


# Paso 01
class MensajeA142(XmlModel):
    _sort_order = ('mensaje', 'heading', 'a142')

    def __init__(self):
        self.doc_root = None
        self.mensaje = XmlField(
            'sctdapplication', attributes={'xmlns': 'http://localhost/sctd/A142'}
        )
        self.heading = Heading()
        self.a142 = A142()
        super(MensajeA142, self).__init__('sctdapplication', 'mensaje')


class A142(XmlModel):

    _sort_order = (
        'a142',
        'comreferencenum',
        'reqdate',
        'reqhour',
        'titulartype',
        'nationality',
        'documenttype',
        'documentnum',
        'cups',
        'modeffectdate',
        'reqtransferdate',
        'updatereason',
        'surrogacy',
        'newowner',
        'disconnectedserviceaccepted',
        'cnae',
        'protectedtype',
        'essentialtype',
        'extrainfo',
        'productlist',
        'registerdoclist',
    )

    def __init__(self):
        self.a142 = XmlField('a142')
        self.comreferencenum = XmlField('comreferencenum')
        self.reqdate = XmlField('reqdate')
        self.reqhour = XmlField('reqhour')
        self.titulartype = XmlField('titulartype')
        self.nationality = XmlField('nationality')
        self.documenttype = XmlField('documenttype')
        self.documentnum = XmlField('documentnum')
        self.cups = XmlField('cups')
        self.modeffectdate = XmlField('modeffectdate')
        self.reqtransferdate = XmlField('reqtransferdate')
        self.updatereason = XmlField('updatereason')
        self.surrogacy = XmlField('surrogacy')
        self.newowner = Newowner()
        self.disconnectedserviceaccepted = XmlField('disconnectedserviceaccepted')
        self.cnae = XmlField('cnae')
        self.protectedtype = XmlField('protectedtype')
        self.essentialtype = XmlField('essentialtype')
        self.extrainfo = XmlField('extrainfo')
        self.productlist = ProductList()
        self.registerdoclist = Registerdoclist()

        super(A142, self).__init__('a142', 'a142')


class ProductList(XmlModel):

    _sort_order = ('productlist', 'product_list')

    def __init__(self):
        self.productlist = XmlField('productlist')
        self.product_list = []
        super(ProductList, self).__init__('productlist', 'productlist')


class Newclient(XmlModel):

    _sort_order = (
        'newnationality',
        'newdocumenttype',
        'newdocumentnum',
        'newfirstname',
        'newfamilyname1',
        'newfamilyname2',
        'newtitulartype',
        'newtelephone1',
        'newtelephone2',
        'newtelephone3',
        'newemail',
        'newlanguage',
    )

    def __init__(self):
        self.newclient = XmlField('newclient')
        self.newnationality = XmlField('newnationality')
        self.newdocumenttype = XmlField('newdocumenttype')
        self.newdocumentnum = XmlField('newdocumentnum')
        self.newfirstname = XmlField('newfirstname')
        self.newfamilyname1 = XmlField('newfamilyname1')
        self.newfamilyname2 = XmlField('newfamilyname2')
        self.newtitulartype = XmlField('newtitulartype')
        self.newtelephone1 = XmlField('newtelephone1')
        self.newtelephone2 = XmlField('newtelephone2')
        self.newtelephone3 = XmlField('newtelephone3')
        self.newemail = XmlField('newemail')
        self.newlanguage = XmlField('newlanguage')
        super(Newclient, self).__init__('newclient','newclient')


class Street(XmlModel):

    _sort_order = (
        'street',
        'streettype',
        'street_name',
        'streetnumber',
        'portal',
        'staircase',
        'floor',
        'door',
        )

    def __init__(self):
        self.street = XmlField('street')
        self.streettype = XmlField('streettype')
        self.street_name = XmlField('street')
        self.streetnumber = XmlField('streetnumber')
        self.portal = XmlField('portal')
        self.staircase = XmlField('staircase')
        self.floor = XmlField('floor')
        self.door = XmlField('door')
        super(Street,self).__init__('street','street')

class Address(XmlModel):

    _sort_order = ('address', 'province', 'city', 'zipcode', 'street')

    def __init__(self):
        self.address = XmlField('address')
        self.province = XmlField('province')
        self.city = XmlField('city')
        self.zipcode = XmlField('zipcode')
        self.street = Street()
        super(Address, self).__init__('address','address')

class AddressPS(XmlModel):

    _sort_order = ('addressPS', 'province', 'city', 'zipcode', 'street')

    def __init__(self):
        self.addressPS = XmlField('addressPS')
        self.province = XmlField('province')
        self.city = XmlField('city')
        self.zipcode = XmlField('zipcode')
        self.street = Street()
        super(AddressPS, self).__init__('addressPS', 'addressPS')


class Newowner(XmlModel):

    _sort_order = ('newowner', 'newclient','newregularaddress','typefiscaladdress','addressPS','address')

    def __init__(self):
        self.newowner = XmlField('newowner')
        self.newclient = Newclient()
        self.newregularaddress = XmlField('newregularaddress')
        self.typefiscaladdress = XmlField('typefiscaladdress')
        self.addressPS = AddressPS()
        self.address = Address()
        super(Newowner, self).__init__('newowner','newowner')

class Product(XmlModel):
    _sort_order = (
    'product', 'reqtype', 'productcode', 'producttype', 'producttolltype', 'productqd', 'productqi', 'productqa', 'productstartdate',
    'productenddate',)

    def __init__(self):
        self.product = XmlField('product')
        self.reqtype = XmlField('reqtype')
        self.productcode = XmlField('productcode')
        self.producttype = XmlField('producttype')
        self.producttolltype = XmlField('producttolltype')
        self.productqd = XmlField('productqd', rep=rep_decimal(7))
        self.productqi = XmlField('productqi', rep=rep_decimal(7))
        self.productqa = XmlField('productqa')
        self.productstartdate = XmlField('productstartdate')
        self.productenddate = XmlField('productenddate')
        super(Product, self).__init__('product', 'product')


# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from libcomxml.core import XmlModel, XmlField
from gestionatr.output.messages.base import Cabecera


# Paso 01
class MensajeCambiodeComercializadorSinCambios(XmlModel):
    _sort_order = ('mensaje', 'cabecera', 'cambiode_comercializador_sin_cambios')

    def __init__(self):
        self.doc_root = None
        self.mensaje = XmlField(
            'MensajeCambiodeComercializadorSinCambios',
            attributes={'xmlns': 'http://localhost/elegibilidad'}
        )
        self.cabecera = Cabecera()
        self.cambiode_comercializador_sin_cambios = CambiodeComercializadorSinCambios()
        super(MensajeCambiodeComercializadorSinCambios, self).__init__(
            'MensajeCambiodeComercializadorSinCambios', 'mensaje'
        )


class CambiodeComercializadorSinCambios(XmlModel):

    _sort_order = ('cambiode_comercializador_sin_cambios', 'datos_solicitud',
                   'cliente', 'comentarios', 'registros_documento')

    def __init__(self):
        self.cambiode_comercializador_sin_cambios = XmlField('CambiodeComercializadorSinCambios')
        self.datos_solicitud = DatosSolicitud()
        self.cliente = Cliente()
        self.comentarios = XmlField('Comentarios')
        self.registros_documento = RegistrosDocumento()
        super(CambiodeComercializadorSinCambios, self).__init__('CambiodeComercializadorSinCambios', 'cambiode_comercializador_sin_cambios')


class DatosSolicitud(XmlModel):

    _sort_order = ('datos_solicitud', 'ind_activacion', 'ind_esencial', 'fecha_ultimo_movimiento_ind_esencial',
                   'fecha_prevista_accion', 'contratacion_incondicional_ps', 'contratacion_incondicional_bs',
                   'bono_social', 'solicitud_tension')

    def __init__(self):
        self.datos_solicitud = XmlField('DatosSolicitud')
        self.ind_activacion = XmlField('IndActivacion')
        self.ind_esencial = XmlField('IndEsencial')
        self.fecha_ultimo_movimiento_ind_esencial = XmlField('FechaUltimoMovimientoIndEsencial')
        self.fecha_prevista_accion = XmlField('FechaPrevistaAccion')
        self.contratacion_incondicional_ps = XmlField('ContratacionIncondicionalPS')
        self.contratacion_incondicional_bs = XmlField('ContratacionIncondicionalBS')
        self.bono_social = XmlField('BonoSocial')
        self.solicitud_tension = XmlField('SolicitudTension')
        super(DatosSolicitud, self).__init__('DatosSolicitud', 'datos_solicitud')


class Cliente(XmlModel):

    _sort_order = ('cliente', 'id_cliente', 'nombre', 'telefonos', 'correo_electronico')

    def __init__(self):
        self.cliente = XmlField('Cliente')
        self.id_cliente = IdCliente()
        self.nombre = Nombre()
        self.telefonos = []
        self.correo_electronico = XmlField('CorreoElectronico')
        super(Cliente, self).__init__('Cliente', 'cliente')


class IdCliente(XmlModel):

    _sort_order = ('id_cliente', 'tipo_identificador', 'identificador', 'tipo_persona')

    def __init__(self, name='IdCliente'):
        self.id_cliente = XmlField(name)
        self.tipo_identificador = XmlField('TipoIdentificador')
        self.identificador = XmlField('Identificador')
        self.tipo_persona = XmlField('TipoPersona')
        super(IdCliente, self).__init__(name, 'id_cliente')


class Nombre(XmlModel):

    _sort_order = ('nombre', 'nombre_de_pila', 'primer_apellido', 'segundo_apellido', 'razon_social')

    def __init__(self):
        self.nombre = XmlField('Nombre')
        self.nombre_de_pila = XmlField('NombreDePila')
        self.primer_apellido = XmlField('PrimerApellido')
        self.segundo_apellido = XmlField('SegundoApellido')
        self.razon_social = XmlField('RazonSocial')
        super(Nombre, self).__init__('Nombre', 'nombre')


class Telefono(XmlModel):

    _sort_order = ('telefono', 'prefijo_pais', 'numero')

    def __init__(self):
        self.telefono = XmlField('Telefono')
        self.prefijo_pais = XmlField('PrefijoPais')
        self.numero = XmlField('Numero')
        super(Telefono, self).__init__('Telefono', 'telefono')


class Autoconsumo(XmlModel):

    _sort_order = ('autoconsumo', 'datos_suministro', 'datos_cau')

    def __init__(self):
        self.autoconsumo = XmlField('Autoconsumo')
        self.datos_suministro = DatosSuministro()
        self.datos_cau = []
        super(Autoconsumo, self).__init__('Autoconsumo', 'autoconsumo')


class DatosCAU(XmlModel):

    _sort_order = ('datos_suministro', 'cau', 'tipo_autoconsumo', 'tipo_subseccion', 'colectivo', 'datos_inst_gen')

    def __init__(self):
        self.datos_cau = XmlField('DatosCAU')
        self.cau = XmlField('CAU')
        self.tipo_autoconsumo = XmlField('TipoAutoconsumo')
        self.tipo_subseccion = XmlField('TipoSubseccion')
        self.colectivo = XmlField('Colectivo')
        self.datos_inst_gen = DatosInstGen()
        super(DatosCAU, self).__init__('DatosCAU', 'datos_cau')


class DatosSuministro(XmlModel):

    _sort_order = ('datos_suministro', 'tipo_cups', 'ref_catastro')

    def __init__(self):
        self.datos_suministro = XmlField('DatosSuministro')
        self.tipo_cups = XmlField('TipoCUPS')
        self.ref_catastro = XmlField('RefCatastro')
        super(DatosSuministro, self).__init__('DatosSuministro', 'datos_suministro')


class DatosInstGen(XmlModel):

    _sort_order = ('datos_inst_gen', 'cil', 'tec_generador', 'combustible', 'pot_instalada_gen', 'tipo_instalacion',
                   'esquema_medida', 'ssaa', 'unico_contrato', 'ref_catastro', 'utm', 'titular_representante_gen')

    def __init__(self):
        self.datos_inst_gen = XmlField('DatosInstGen')
        self.cil = XmlField('CIL')
        self.tec_generador = XmlField('TecGenerador')
        self.combustible = XmlField('Combustible')
        self.pot_instalada_gen = XmlField('PotInstaladaGen')
        self.tipo_instalacion = XmlField('TipoInstalacion')
        self.esquema_medida = XmlField('EsquemaMedida')
        self.ssaa = XmlField('SSAA')
        self.unico_contrato = XmlField('UnicoContrato')
        self.ref_catastro = XmlField('RefCatastro')
        self.utm = UTM()
        self.titular_representante_gen = TitularRepresentanteGen()
        super(DatosInstGen, self).__init__('DatosInstGen', 'datos_inst_gen')


class UTM(XmlModel):

    _sort_order = ('utm', 'x', 'y', 'huso', 'banda')

    def __init__(self):
        self.utm = XmlField('UTM')
        self.x = XmlField('X')
        self.y = XmlField('Y')
        self.huso = XmlField('Huso')
        self.banda = XmlField('Banda')
        super(UTM, self).__init__('UTM', 'utm')


class TitularRepresentanteGen(XmlModel):

    _sort_order = ('titular_representante_gen', 'id_titular', 'nombre', 'telefono', 'correo_electronico')

    def __init__(self):
        self.titular_representante_gen = XmlField('TitularRepresentanteGen')
        self.id_titular = IdTitular()
        self.nombre = Nombre()
        self.telefono = Telefono()
        self.correo_electronico = XmlField('CorreoElectronico')
        super(TitularRepresentanteGen, self).__init__('TitularRepresentanteGen', 'titular_representante_gen')


class IdTitular(XmlModel):

    _sort_order = ('id_titular', 'tipo_identificador', 'identificador')

    def __init__(self):
        self.id_titular = XmlField('IdTitular')
        self.tipo_identificador = XmlField('TipoIdentificador')
        self.identificador = XmlField('Identificador')
        super(IdTitular, self).__init__('IdTitular', 'id_titular')


class RegistrosDocumento(XmlModel):

    _sort_order = ('registros_documento', 'registro_doc_list')

    def __init__(self):
        self.registros_documento = XmlField('RegistrosDocumento')
        self.registro_doc_list = []
        super(RegistrosDocumento, self).__init__('RegistrosDocumento', 'registros_documento')


class RegistroDoc(XmlModel):

    _sort_order = ('registro_doc', 'tipo_doc_aportado', 'direccion_url')

    def __init__(self):
        self.registro_doc = XmlField('RegistroDoc')
        self.tipo_doc_aportado = XmlField('TipoDocAportado')
        self.direccion_url = XmlField('DireccionUrl')
        super(RegistroDoc, self).__init__('RegistroDoc', 'registro_doc')


# Paso 02 accept
class MensajeAceptacionCambiodeComercializadorSinCambios(XmlModel):

    _sort_order = ('mensaje', 'cabecera', 'aceptacion_cambiode_comercializador_sin_cambios')

    def __init__(self):
        self.mensaje = XmlField('MensajeAceptacionCambiodeComercializadorSinCambios',
                                attributes={'xmlns': 'http://localhost/elegibilidad'})
        self.cabecera = Cabecera()
        self.aceptacion_cambiode_comercializador_sin_cambios = AceptacionCambiodeComercializadorSinCambios()
        super(MensajeAceptacionCambiodeComercializadorSinCambios, self).__init__('MensajeAceptacionCambiodeComercializadorSinCambios', 'mensaje')


class AceptacionCambiodeComercializadorSinCambios(XmlModel):

    _sort_order = ('aceptacion_cambiode_comercializador_sin_cambios', 'datos_aceptacion', 'contrato')

    def __init__(self):
        self.aceptacion_cambiode_comercializador_sin_cambios = XmlField('AceptacionCambiodeComercializadorSinCambios')
        self.datos_aceptacion = DatosAceptacion()
        self.contrato = Contrato()
        super(AceptacionCambiodeComercializadorSinCambios, self).__init__('AceptacionCambiodeComercializadorSinCambios', 'aceptacion_cambiode_comercializador_sin_cambios')


class DatosAceptacion(XmlModel):

    _sort_order = ('datos_aceptacion', 'fecha_aceptacion', 'potencia_actual', 'actuacion_campo', 'fecha_ultima_lectura_firme', 'bono_social')

    def __init__(self):
        self.datos_aceptacion = XmlField('DatosAceptacion')
        self.fecha_aceptacion = XmlField('FechaAceptacion')
        self.potencia_actual = XmlField('PotenciaActual')
        self.actuacion_campo = XmlField('ActuacionCampo')
        self.fecha_ultima_lectura_firme = XmlField('FechaUltimaLecturaFirme')
        self.bono_social = XmlField('BonoSocial')
        super(DatosAceptacion, self).__init__('DatosAceptacion', 'datos_aceptacion')


class Contrato(XmlModel):

    _sort_order = ('contrato', 'id_contrato', 'data_finalitzacio', 'autoconsumo', 'tipo_contrato_atr', 'condiciones_contractuales', 'tipo_activacion_prevista', 'fecha_activacion_prevista')

    def __init__(self):
        self.contrato = XmlField('Contrato')
        self.id_contrato = IdContrato()
        self.data_finalitzacio = XmlField('FechaFinalizacion')
        self.autoconsumo = Autoconsumo()
        self.tipo_contrato_atr = XmlField('TipoContratoATR')
        self.condiciones_contractuales = CondicionesContractuales()
        self.tipo_activacion_prevista = XmlField('TipoActivacionPrevista')
        self.fecha_activacion_prevista = XmlField('FechaActivacionPrevista')
        super(Contrato, self).__init__('Contrato', 'contrato')


class IdContrato(XmlModel):

    _sort_order = ('id_contrato', 'cod_contrato')

    def __init__(self):
        self.id_contrato = XmlField('IdContrato')
        self.cod_contrato = XmlField('CodContrato')
        super(IdContrato, self).__init__('IdContrato', 'id_contrato')


class CondicionesContractuales(XmlModel):

    _sort_order = ('condiciones_contractuales', 'tarifa_atr', 'periodicidad_facturacion', 'tipode_telegestion', 'potencias_contratadas', 'modo_control_potencia', 'marca_medida_con_perdidas', 'tension_del_suministro', 'vas_trafo', 'porcentaje_perdidas')

    def __init__(self):
        self.condiciones_contractuales = XmlField('CondicionesContractuales')
        self.tarifa_atr = XmlField('TarifaATR')
        self.periodicidad_facturacion = XmlField('PeriodicidadFacturacion')
        self.tipode_telegestion = XmlField('TipodeTelegestion')
        self.potencias_contratadas = PotenciasContratadas()
        self.modo_control_potencia = XmlField('ModoControlPotencia')
        self.marca_medida_con_perdidas = XmlField('MarcaMedidaConPerdidas')
        self.tension_del_suministro = XmlField('TensionDelSuministro')
        self.vas_trafo = XmlField('VAsTrafo')
        self.porcentaje_perdidas = XmlField('PorcentajePerdidas')
        super(CondicionesContractuales, self).__init__('CondicionesContractuales', 'condiciones_contractuales')


class PotenciasContratadas(XmlModel):
    _sort_order = ('potencies', 'p1', 'p2', 'p3', 'p4', 'p5', 'p6', 'p7', 'p8',
                   'p9', 'p10')

    def __init__(self):
        self.potencies = XmlField('PotenciasContratadas')
        self.p1 = XmlField('Potencia', attributes={'Periodo': '1'}, rep=lambda x: '%d' % x)
        self.p2 = XmlField('Potencia', attributes={'Periodo': '2'})
        self.p3 = XmlField('Potencia', attributes={'Periodo': '3'})
        self.p4 = XmlField('Potencia', attributes={'Periodo': '4'})
        self.p5 = XmlField('Potencia', attributes={'Periodo': '5'})
        self.p6 = XmlField('Potencia', attributes={'Periodo': '6'})
        self.p7 = XmlField('Potencia', attributes={'Periodo': '7'})
        self.p8 = XmlField('Potencia', attributes={'Periodo': '8'})
        self.p9 = XmlField('Potencia', attributes={'Periodo': '9'})
        self.p10 = XmlField('Potencia', attributes={'Periodo': '10'})
        super(PotenciasContratadas, self).__init__('PotenciasContratadas', 'potencies')


# Paso 04 (Rechazo)
class MensajeRechazo(XmlModel):

    _sort_order = ('mensaje_rechazo', 'cabecera', 'rechazos')

    def __init__(self):
        self.mensaje_rechazo = XmlField('MensajeRechazo',
                                        attributes={'xmlns': 'http://localhost/elegibilidad'})
        self.cabecera = Cabecera()
        self.rechazos = Rechazos()
        super(MensajeRechazo, self).__init__('MensajeRechazo', 'mensaje_rechazo')


class Rechazos(XmlModel):

    _sort_order = ('rechazos', 'fecha_rechazo', 'rechazo_list', 'registros_documento')

    def __init__(self):
        self.rechazos = XmlField('Rechazos')
        self.fecha_rechazo = XmlField('FechaRechazo')
        self.rechazo_list = []
        self.registros_documento = RegistrosDocumento()
        super(Rechazos, self).__init__('Rechazos', 'rechazos')


class Rechazo(XmlModel):

    _sort_order = ('rechazo', 'secuencial', 'codigo_motivo', 'comentarios')

    def __init__(self):
        self.rechazo = XmlField('Rechazo')
        self.secuencial = XmlField('Secuencial')
        self.codigo_motivo = XmlField('CodigoMotivo')
        self.comentarios = XmlField('Comentarios')
        super(Rechazo, self).__init__('Rechazo', 'rechazo')


# Paso 05
class MensajeActivacionCambiodeComercializadorSinCambios(XmlModel):

    _sort_order = ('mensaje', 'cabecera', 'activacion_cambiode_comercializador_sin_cambios')

    def __init__(self):
        self.mensaje = XmlField('MensajeActivacionCambiodeComercializadorSinCambios',
                                attributes={'xmlns': 'http://localhost/elegibilidad'})
        self.cabecera = Cabecera()
        self.activacion_cambiode_comercializador_sin_cambios = ActivacionCambiodeComercializadorSinCambios()
        super(MensajeActivacionCambiodeComercializadorSinCambios, self).__init__('MensajeActivacionCambiodeComercializadorSinCambios', 'mensaje')


class ActivacionCambiodeComercializadorSinCambios(XmlModel):

    _sort_order = ('activacion_cambiode_comercializador_sin_cambios', 'datos_activacion', 'contrato',
                   'puntos_de_medida', 'registros_documento')

    def __init__(self):
        self.activacion_cambiode_comercializador_sin_cambios = XmlField('ActivacionCambiodeComercializadorSinCambios')
        self.datos_activacion = DatosActivacion()
        self.contrato = Contrato()
        self.puntos_de_medida = PuntosDeMedida()
        self.registros_documento = RegistrosDocumento()
        super(ActivacionCambiodeComercializadorSinCambios, self).__init__('ActivacionCambiodeComercializadorSinCambios', 'activacion_cambiode_comercializador_sin_cambios')


class DatosActivacion(XmlModel):

    _sort_order = ('datos_activacion', 'fecha', 'bono_social', 'ind_esencial', 'fecha_ultimo_movimiento_ind_esencial')

    def __init__(self):
        self.datos_activacion = XmlField('DatosActivacion')
        self.fecha = XmlField('Fecha')
        self.bono_social = XmlField('BonoSocial')
        self.ind_esencial = XmlField('IndEsencial')
        self.fecha_ultimo_movimiento_ind_esencial = XmlField('FechaUltimoMovimientoIndEsencial')
        super(DatosActivacion, self).__init__('DatosActivacion', 'datos_activacion')


class PuntosDeMedida(XmlModel):

    _sort_order = ('puntos_de_medida', 'punto_de_medida_list')

    def __init__(self):
        self.puntos_de_medida = XmlField('PuntosDeMedida')
        self.punto_de_medida_list = []
        super(PuntosDeMedida, self).__init__('PuntosDeMedida', 'puntos_de_medida')


class PuntoDeMedida(XmlModel):

    _sort_order = ('punto_de_medida', 'cod_pm', 'tipo_movimiento', 'tipo_pm', 'cod_pm_principal', 'modo_lectura', 'funcion', 'direccion_enlace', 'direccion_punto_medida', 'num_linea', 'telefono_telemedida', 'estado_telefono', 'clave_acceso', 'tension_pm', 'fecha_vigor', 'fecha_alta', 'fecha_baja', 'aparatos', 'comentarios')

    def __init__(self):
        self.punto_de_medida = XmlField('PuntoDeMedida')
        self.cod_pm = XmlField('CodPM')
        self.tipo_movimiento = XmlField('TipoMovimiento')
        self.tipo_pm = XmlField('TipoPM')
        self.cod_pm_principal = XmlField('CodPMPrincipal')
        self.modo_lectura = XmlField('ModoLectura')
        self.funcion = XmlField('Funcion')
        self.direccion_enlace = XmlField('DireccionEnlace')
        self.direccion_punto_medida = XmlField('DireccionPuntoMedida')
        self.num_linea = XmlField('NumLinea')
        self.telefono_telemedida = XmlField('TelefonoTelemedida')
        self.estado_telefono = XmlField('EstadoTelefono')
        self.clave_acceso = XmlField('ClaveAcceso')
        self.tension_pm = XmlField('TensionPM')
        self.fecha_vigor = XmlField('FechaVigor')
        self.fecha_alta = XmlField('FechaAlta')
        self.fecha_baja = XmlField('FechaBaja')
        self.aparatos = Aparatos()
        self.comentarios = XmlField('Comentarios')
        super(PuntoDeMedida, self).__init__('PuntoDeMedida', 'punto_de_medida')


class Aparatos(XmlModel):

    _sort_order = ('aparatos', 'aparato_list')

    def __init__(self):
        self.aparatos = XmlField('Aparatos')
        self.aparato_list = []
        super(Aparatos, self).__init__('Aparatos', 'aparatos')


class Aparato(XmlModel):

    _sort_order = ('aparato', 'modelo_aparato', 'tipo_movimiento', 'tipo_equipo_medida', 'tipo_propiedad_aparato', 'propietario', 'tipo_dhedm', 'modo_medida_potencia', 'lectura_directa', 'cod_precinto', 'datos_aparato', 'medidas')

    def __init__(self):
        self.aparato = XmlField('Aparato')
        self.modelo_aparato = ModeloAparato()
        self.tipo_movimiento = XmlField('TipoMovimiento')
        self.tipo_equipo_medida = XmlField('TipoEquipoMedida')
        self.tipo_propiedad_aparato = XmlField('TipoPropiedadAparato')
        self.propietario = XmlField('Propietario')
        self.tipo_dhedm = XmlField('TipoDHEdM')
        self.modo_medida_potencia = XmlField('ModoMedidaPotencia')
        self.lectura_directa = XmlField('LecturaDirecta')
        self.cod_precinto = XmlField('CodPrecinto')
        self.datos_aparato = DatosAparato()
        self.medidas = Medidas()
        super(Aparato, self).__init__('Aparato', 'aparato')


class ModeloAparato(XmlModel):

    _sort_order = ('modelo_aparato', 'tipo_aparato', 'marca_aparato', 'modelo_marca')

    def __init__(self):
        self.modelo_aparato = XmlField('ModeloAparato')
        self.tipo_aparato = XmlField('TipoAparato')
        self.marca_aparato = XmlField('MarcaAparato')
        self.modelo_marca = XmlField('ModeloMarca')
        super(ModeloAparato, self).__init__('ModeloAparato', 'modelo_aparato')


class DatosAparato(XmlModel):

    _sort_order = ('datos_aparato', 'periodo_fabricacion', 'numero_serie', 'funcion_aparato', 'num_integradores', 'constante_energia', 'constante_maximetro', 'ruedas_enteras', 'ruedas_decimales')

    def __init__(self):
        self.datos_aparato = XmlField('DatosAparato')
        self.periodo_fabricacion = XmlField('PeriodoFabricacion')
        self.numero_serie = XmlField('NumeroSerie')
        self.funcion_aparato = XmlField('FuncionAparato')
        self.num_integradores = XmlField('NumIntegradores')
        self.constante_energia = XmlField('ConstanteEnergia')
        self.constante_maximetro = XmlField('ConstanteMaximetro')
        self.ruedas_enteras = XmlField('RuedasEnteras')
        self.ruedas_decimales = XmlField('RuedasDecimales')
        super(DatosAparato, self).__init__('DatosAparato', 'datos_aparato')


class Medidas(XmlModel):

    _sort_order = ('medidas', 'medida_list')

    def __init__(self):
        self.medidas = XmlField('Medidas')
        self.medida_list = []
        super(Medidas, self).__init__('Medidas', 'medidas')


class Medida(XmlModel):

    _sort_order = ('medida', 'tipo_dhedm', 'periodo', 'magnitud_medida', 'procedencia', 'ultima_lectura_firme', 'fecha_lectura_firme', 'anomalia', 'comentarios')

    def __init__(self, drop_empty=False):
        self.medida = XmlField('Medida')
        self.tipo_dhedm = XmlField('TipoDHEdM')
        self.periodo = XmlField('Periodo')
        self.magnitud_medida = XmlField('MagnitudMedida')
        self.procedencia = XmlField('Procedencia')
        self.ultima_lectura_firme = XmlField('UltimaLecturaFirme')
        self.fecha_lectura_firme = XmlField('FechaLecturaFirme')
        self.anomalia = XmlField('Anomalia')
        self.comentarios = XmlField('Comentarios')
        super(Medida, self).__init__('Medida', 'medida')


# Paso 06
class MensajeActivacionComercializadorSaliente(XmlModel):

    _sort_order = ('mensaje', 'cabecera', 'notificacion_comercializador_saliente')

    def __init__(self):
        self.mensaje = XmlField('MensajeActivacionComercializadorSaliente',
                                attributes={'xmlns': 'http://localhost/elegibilidad'})
        self.cabecera = Cabecera()
        self.notificacion_comercializador_saliente = NotificacionComercializadorSaliente()
        super(MensajeActivacionComercializadorSaliente, self).__init__('MensajeActivacionComercializadorSaliente', 'mensaje')


class NotificacionComercializadorSaliente(XmlModel):

    _sort_order = ('notificacion_comercializador_saliente', 'datos_notificacion', 'contrato', 'puntos_de_medida')

    def __init__(self):
        self.notificacion_comercializador_saliente = XmlField('NotificacionComercializadorSaliente')
        self.datos_notificacion = DatosNotificacion()
        self.contrato = Contrato()
        self.puntos_de_medida = PuntosDeMedida()
        super(NotificacionComercializadorSaliente, self).__init__('NotificacionComercializadorSaliente', 'notificacion_comercializador_saliente')


class DatosNotificacion(XmlModel):

    _sort_order = ('datos_notificacion', 'fecha_activacion', 'ind_bono_social')

    def __init__(self):
        self.datos_notificacion = XmlField('DatosNotificacion')
        self.fecha_activacion = XmlField('FechaActivacion')
        self.ind_bono_social = XmlField('IndBonoSocial')
        super(DatosNotificacion, self).__init__('DatosNotificacion', 'datos_notificacion')


# Paso 08
class MensajeAnulacionSolicitud(XmlModel):

    _sort_order = ('mensaje', 'cabecera')

    def __init__(self):
        self.mensaje = XmlField('MensajeAnulacionSolicitud',
                                attributes={'xmlns': 'http://localhost/elegibilidad'})
        self.cabecera = Cabecera()
        super(MensajeAnulacionSolicitud, self).__init__('MensajeAnulacionSolicitud', 'mensaje')


# Paso 09, 10
class MensajeAceptacionAnulacion(XmlModel):

    _sort_order = ('mensaje', 'cabecera', 'aceptacion_anulacion')

    def __init__(self):
        self.mensaje = XmlField('MensajeAceptacionAnulacion',
                                attributes={'xmlns': 'http://localhost/elegibilidad'})
        self.cabecera = Cabecera()
        self.aceptacion_anulacion = AceptacionAnulacion()
        super(MensajeAceptacionAnulacion, self).__init__('MensajeAceptacionAnulacion', 'mensaje')


class AceptacionAnulacion(XmlModel):

    _sort_order = ('aceptacion_anulacion', 'fecha_aceptacion')

    def __init__(self):
        self.aceptacion_anulacion = XmlField('AceptacionAnulacion')
        self.fecha_aceptacion = XmlField('FechaAceptacion')
        super(AceptacionAnulacion, self).__init__('AceptacionAnulacion', 'aceptacion_anulacion')


# Paso 11
class MensajeAceptacionCambiodeComercializadorSaliente(XmlModel):

    _sort_order = ('mensaje', 'cabecera', 'aceptacion_cambio_comercializador_saliente')

    def __init__(self):
        self.mensaje = XmlField('MensajeAceptacionCambiodeComercializadorSaliente',
                                attributes={'xmlns': 'http://localhost/elegibilidad'})
        self.cabecera = Cabecera()
        self.aceptacion_cambio_comercializador_saliente = AceptacionCambioComercializadorSaliente()
        super(MensajeAceptacionCambiodeComercializadorSaliente, self).__init__('MensajeAceptacionCambiodeComercializadorSaliente', 'mensaje')


class AceptacionCambioComercializadorSaliente(XmlModel):

    _sort_order = ('aceptacion_cambio_comercializador_saliente', 'fecha_activacion_prevista', 'ind_bono_social')

    def __init__(self):
        self.aceptacion_cambio_comercializador_saliente = XmlField('AceptacionCambioComercializadorSaliente')
        self.fecha_activacion_prevista = XmlField('FechaActivacionPrevista')
        self.ind_bono_social = XmlField('IndBonoSocial')
        super(AceptacionCambioComercializadorSaliente, self).__init__('AceptacionCambioComercializadorSaliente', 'aceptacion_cambio_comercializador_saliente')


# Paso 12
class MensajeRechazoCambiodeComercializadorSaliente(XmlModel):

    _sort_order = ('mensaje', 'cabecera', 'rechazo_cambio_comercializador_saliente')

    def __init__(self):
        self.mensaje = XmlField('MensajeRechazoCambiodeComercializadorSaliente',
                                attributes={'xmlns': 'http://localhost/elegibilidad'})
        self.cabecera = Cabecera()
        self.rechazo_cambio_comercializador_saliente = RechazoCambioComercializadorSaliente()
        super(MensajeRechazoCambiodeComercializadorSaliente, self).__init__('MensajeRechazoCambiodeComercializadorSaliente', 'mensaje')


class RechazoCambioComercializadorSaliente(XmlModel):

    _sort_order = ('rechazo_cambio_comercializador_saliente', 'fecha_rechazo')

    def __init__(self):
        self.rechazo_cambio_comercializador_saliente = XmlField('RechazoCambioComercializadorSaliente')
        self.fecha_rechazo = XmlField('FechaRechazo')
        super(RechazoCambioComercializadorSaliente, self).__init__('RechazoCambioComercializadorSaliente', 'rechazo_cambio_comercializador_saliente')



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from gestionatr.input.messages import C1
from gestionatr.input.messages.C1 import Autoconsumo
from gestionatr.utils import get_rec_attr


class D1(C1):
    """Clase que implementa D1."""

    @property
    def notificacion_cambios_atr_desde_distribuidor(self):
        tree = 'NotificacionCambiosATRDesdeDistribuidor'
        data = []
        for d in get_rec_attr(self.obj, tree, False):
            data.append(NotificacionCambiosATRDesdeDistribuidor(d))
        return data

    @property
    def anulacion_notificacion_cambios_atr_desde_distribuidor(self):
        tree = 'AnulacionNotificacionCambiosATRDesdeDistribuidor'
        data = get_rec_attr(self.obj, tree, False)
        if data not in [None, False]:
            return AnulacionNotificacionCambiosATRDesdeDistribuidor(data)
        else:
            return False


class NotificacionCambiosATRDesdeDistribuidor(object):

    def __init__(self, data):
        self.notificacion_cambios_atr_desde_distribuidor = data

    @property
    def motivo_cambio_atr_desde_distribuidora(self):
        data = ''
        try:
            data = self.notificacion_cambios_atr_desde_distribuidor.MotivoCambioATRDesdeDistribuidora.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_prevista_aplicacion_cambio_atr(self):
        data = ''
        try:
            data = self.notificacion_cambios_atr_desde_distribuidor.FechaPrevistaAplicacionCambioATR.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_maxima_rechazo(self):
        data = ''
        try:
            data = self.notificacion_cambios_atr_desde_distribuidor.FechaMaximaRechazo.text
        except AttributeError:
            pass
        return data

    @property
    def periodicidad_facturacion(self):
        data = ''
        try:
            data = self.notificacion_cambios_atr_desde_distribuidor.PeriodicidadFacturacion.text
        except AttributeError:
            pass
        return data

    @property
    def ind_esencial(self):
        data = False
        try:
            data = self.notificacion_cambios_atr_desde_distribuidor.IndEsencial.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_ultimo_movimiento_ind_esencial(self):
        data = False
        try:
            data = self.notificacion_cambios_atr_desde_distribuidor.FechaUltimoMovimientoIndEsencial.text
        except AttributeError:
            pass
        return data

    @property
    def info_registro_autocons(self):
        data = ''
        try:
            data = InfoRegistroAutocons(self.notificacion_cambios_atr_desde_distribuidor.InfoRegistroAutocons)
        except AttributeError:
            pass
        return data

    @property
    def info_retardo_activ_autocons(self):
        data = []
        try:
            for datos in self.notificacion_cambios_atr_desde_distribuidor.InfoRetardoActivAutocons:
                data.append(InfoRetardoActivAutocons(datos))
        except AttributeError:
            pass
        return data


class AnulacionNotificacionCambiosATRDesdeDistribuidor(object):

    def __init__(self, data):
        self.anulacion_notificacion_cambios_atr_desde_distribuidor = data

    @property
    def datos_anulacion(self):
        data = ''
        try:
            data = DatosAnulacion(self.anulacion_notificacion_cambios_atr_desde_distribuidor.DatosAnulacion)
        except AttributeError:
            pass
        return data


class DatosAnulacion(object):

    def __init__(self, data):
        self.datos_anulacion = data

    @property
    def fecha_anulacion(self):
        data = ''
        try:
            data = self.datos_anulacion.FechaAnulacion.text
        except AttributeError:
            pass
        return data


class InfoRetardoActivAutocons(object):

    def __init__(self, data):
        self.info_retardo_activ_atuocons = data

    @property
    def codigo_fiscal_factura(self):
        data = ''
        try:
            data = self.info_retardo_activ_atuocons.CodigoFiscalFactura.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_inicio_conteo_activ_autocons(self):
        data = ''
        try:
            data = self.info_retardo_activ_atuocons.FechaInicioConteoActivAutocons.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_desde(self):
        data = ''
        try:
            data = self.info_retardo_activ_atuocons.FechaDesde.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_hasta(self):
        data = ''
        try:
            data = self.info_retardo_activ_atuocons.FechaHasta.text
        except AttributeError:
            pass
        return data

    @property
    def dias_retardo_activ_autocons(self):
        data = ''
        try:
            data = self.info_retardo_activ_atuocons.DiasRetardoActivAutocons.text
        except AttributeError:
            pass
        return data

    @property
    def valor_energia_anual_calculado(self):
        data = ''
        try:
            data = self.info_retardo_activ_atuocons.ValorEnergiaAnualCalculada.text
        except AttributeError:
            pass
        return data

    @property
    def valor_energia_horaria_calculada(self):
        data = ''
        try:
            data = self.info_retardo_activ_atuocons.ValorEnergiaHorariaCalculada.text
        except AttributeError:
            pass
        return data

    @property
    def pot_instalada_gen(self):
        data = ''
        try:
            data = self.info_retardo_activ_atuocons.PotInstaladaGen.text
        except AttributeError:
            pass
        return data


class InfoRegistroAutocons(object):

    def __init__(self, data):
        self.info_registro_autocons = data

    @property
    def autoconsumo(self):
        data = ''
        try:
            data = Autoconsumo(self.info_registro_autocons.Autoconsumo)
        except AttributeError:
            pass
        return data



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from .message_gas import MessageGas
from .Deadlines import ProcessDeadline, DeadLine, Workdays, Naturaldays
from gestionatr.utils import get_rec_attr


class A1_41(MessageGas, ProcessDeadline):
    """Clase que implementa C2."""

    steps = []

    # Datos paso a141
    # TODO

    # Datos paso a241
    @property
    def reqcode(self):
        tree = '{0}.reqcode'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def responsedate(self):
        tree = '{0}.responsedate'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def responsehour(self):
        tree = '{0}.responsehour'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def comreferencenum(self):
        tree = '{0}.comreferencenum'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def result(self):
        tree = '{0}.result'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def resultdesc(self):
        tree = '{0}.resultdesc'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def resultreason(self):
        tree = '{0}.resultreason'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def resultreasondesc(self):
        tree = '{0}.resultreasondesc'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def nationality(self):
        tree = '{0}.nationality'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def documenttype(self):
        tree = '{0}.documenttype'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def documentnum(self):
        tree = '{0}.documentnum'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def updatereason(self):
        tree = '{0}.updatereason'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def reqqd(self):
        tree = '{0}.reqqd'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def reqqh(self):
        tree = '{0}.reqqh'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def reqestimatedqa(self):
        tree = '{0}.reqestimatedqa'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def reqoutgoingpressure(self):
        tree = '{0}.reqoutgoingpressure'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def tolltype(self):
        tree = '{0}.tolltype'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def qdgranted(self):
        tree = '{0}.qdgranted'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def singlenomination(self):
        tree = '{0}.singlenomination'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def netsituation(self):
        tree = '{0}.netsituation'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def newmodeffectdate(self):
        tree = '{0}.newmodeffectdate'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def foreseentransferdate(self):
        tree = '{0}.foreseentransferdate'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def StatusPS(self):
        tree = '{0}.StatusPS'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def extrainfo(self):
        tree = '{0}.extrainfo'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    # Datos paso a2541
    @property
    def visitdate(self):
        tree = '{0}.visitdate'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def visithour(self):
        tree = '{0}.visithour'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def informationtype(self):
        tree = '{0}.informationtype'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def informationtypedesc(self):
        tree = '{0}.informationtypedesc'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def operationnum(self):
        tree = '{0}.operationnum'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def defectlist(self):
        data = []
        obj = get_rec_attr(self.obj, self._header, False)
        if (hasattr(obj, 'defectlist') and
                hasattr(obj.defectlist, 'defect')):
            for d in obj.defectlist.defect:
                data.append(defect(d))
        return data

    # Datos paso a341
    @property
    def atrcode(self):
        tree = '{0}.atrcode'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def transfereffectivedate(self):
        tree = '{0}.transfereffectivedate'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def telemetering(self):
        tree = '{0}.telemetering'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def factmethod(self):
        tree = '{0}.factmethod'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def cnae(self):
        tree = '{0}.cnae'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def protectedtype(self):
        tree = '{0}.protectedtype'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def essentialtype(self):
        tree = '{0}.essentialtype'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def gasstationtype(self):
        tree = '{0}.gasstationtype'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def finalqdgranted(self):
        tree = '{0}.finalqdgranted'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def finalqhgranted(self):
        tree = '{0}.finalqhgranted'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def finalclientyearlyconsumption(self):
        tree = '{0}.finalclientyearlyconsumption'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def clientyearlyconsumption(self):
        tree = '{0}.clientyearlyconsumption'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def gasusetype(self):
        tree = '{0}.gasusetype'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def activationtype(self):
        tree = '{0}.activationtype'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def activationtypedesc(self):
        tree = '{0}.activationtypedesc'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def closingtype(self):
        tree = '{0}.closingtype'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def closingtypedesc(self):
        tree = '{0}.closingtypedesc'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def interventiondate(self):
        tree = '{0}.interventiondate'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def interventionhourfrom(self):
        tree = '{0}.interventionhourfrom'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def interventionhourto(self):
        tree = '{0}.interventionhourto'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def visitnumber(self):
        tree = '{0}.visitnumber'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def firstname(self):
        tree = '{0}.firstname'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def familyname1(self):
        tree = '{0}.familyname1'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def familyname2(self):
        tree = '{0}.familyname2'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def titulartype(self):
        tree = '{0}.titulartype'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def regularaddress(self):
        tree = '{0}.regularaddress'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def telephone1(self):
        tree = '{0}.telephone1'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def telephone2(self):
        tree = '{0}.telephone2'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def email(self):
        tree = '{0}.email'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def language(self):
        tree = '{0}.language'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def provinceowner(self):
        tree = '{0}.provinceowner'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def cityowner(self):
        tree = '{0}.cityowner'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def zipcodeowner(self):
        tree = '{0}.zipcodeowner'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def streettypeowner(self):
        tree = '{0}.streettypeowner'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def streetowner(self):
        tree = '{0}.streetowner'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def streetnumberowner(self):
        tree = '{0}.streetnumberowner'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def portalowner(self):
        tree = '{0}.portalowner'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def staircaseowner(self):
        tree = '{0}.staircaseowner'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def floorowner(self):
        tree = '{0}.floorowner'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def doorowner(self):
        tree = '{0}.doorowner'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def canonircperiodicity(self):
        tree = '{0}.canonircperiodicity'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def lastinspectionsdate(self):
        tree = '{0}.lastinspectionsdate'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def lastinspectionsresult(self):
        tree = '{0}.lastinspectionsresult'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False


    @property
    def readingtype(self):
        tree = '{0}.readingtype'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def lectureperiodicity(self):
        tree = '{0}.lectureperiodicity'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def counterlist(self):
        data = []
        obj = get_rec_attr(self.obj, self._header, False)
        if (hasattr(obj, 'counterlist') and
                hasattr(obj.counterlist, 'counter')):
            for d in obj.counterlist.counter:
                data.append(counter(d))
        return data

    @property
    def correctorlist(self):
        data = []
        obj = get_rec_attr(self.obj, self._header, False)
        if (hasattr(obj, 'correctorlist') and
                hasattr(obj.correctorlist, 'corrector')):
            for d in obj.correctorlist.corrector:
                data.append(corrector(d))
        return data

    @property
    def productlist(self):
        data = []
        obj = get_rec_attr(self.obj, self._header, False)
        if (hasattr(obj, 'productlist') and
                hasattr(obj.productlist, 'product')):
            for d in obj.productlist.product:
                data.append(product(d))
        return data

    # Datos paso a441
    # Ya está soportado con los atributos que tenemos

    # Datos paso a3s41
    @property
    def previousatrcode(self):
        tree = '{0}.previousatrcode'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False


class defect(object):

    def __init__(self, data):
        self.obj = data

    @property
    def code(self):
        tree = 'code'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def description(self):
        tree = 'description'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False


class counter(object):

    def __init__(self, data):
        self.obj = data

    @property
    def countermodel(self):
        tree = 'countermodel'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def countertype(self):
        tree = 'countertype'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def counternumber(self):
        tree = 'counternumber'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def counterproperty(self):
        tree = 'counterproperty'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def reallecture(self):
        tree = 'reallecture'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def counterpressure(self):
        tree = 'counterpressure'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False


class corrector(object):

    def __init__(self, data):
        self.obj = data

    @property
    def correctormodel(self):
        tree = 'correctormodel'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def correctortype(self):
        tree = 'correctortype'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def correctornumber(self):
        tree = 'correctornumber'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def correctorproperty(self):
        tree = 'correctorproperty'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def correctedlecture(self):
        tree = 'correctedlecture'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False


class product(object):

    def __init__(self, data):
        self.obj = data

    @property
    def productcode(self):
        tree = 'productcode'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def producttype(self):
        tree = 'producttype'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def producttolltype(self):
        tree = 'producttolltype'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def productqd(self):
        tree = 'productqd'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def productqa(self):
        tree = 'productqa'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def productstartdate(self):
        tree = 'productstartdate'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def productenddate(self):
        tree = 'productenddate'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False

    @property
    def productqi(self):
        tree = 'productqi'
        data = get_rec_attr(self.obj, tree, False)
        if data is not None and data is not False:
            return data.text
        else:
            return False





# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from .C2 import C2, Direccion
from gestionatr.utils import get_rec_attr


class T1(C2):
    """Classe que implementa B1."""

    # Datos paso 01
    @property
    def datos_solicitud(self):
        tree = '{0}.DatosSolicitud'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data not in [None, False]:
            return DatosSolicitud(data)
        else:
            return False

    @property
    def direccion_ps(self):
        tree = '{0}.DireccionPS'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data not in [None, False]:
            return Direccion(data)
        else:
            return False

    # Datos paso 05
    @property
    def datos_activacion(self):
        tree = '{0}.DatosActivacion'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data not in [None, False]:
            return DatosActivacion(data)
        else:
            return False

    # Datos paso 06
    @property
    def datos_notificacion(self):
        tree = '{0}.DatosNotificacion'.format(self._header)
        data = get_rec_attr(self.obj, tree, False)
        if data not in [None, False]:
            return DatosNotificacion(data)
        else:
            return False


class DatosSolicitud(object):

    def __init__(self, data):
        self.datos_solicitud = data

    @property
    def motivo_traspaso(self):
        data = ''
        try:
            data = self.datos_solicitud.MotivoTraspaso.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_prevista_accion(self):
        data = ''
        try:
            data = self.datos_solicitud.FechaPrevistaAccion.text
        except AttributeError:
            pass
        return data

    @property
    def cnae(self):
        data = ''
        try:
            data = self.datos_solicitud.CNAE.text
        except AttributeError:
            pass
        return data

    @property
    def ind_esencial(self):
        data = ''
        try:
            data = self.datos_solicitud.IndEsencial.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_ultimo_movimiento_ind_esencial(self):
        data = False
        try:
            data = self.datos_solicitud.FechaUltimoMovimientoIndEsencial.text
        except AttributeError:
            pass
        return data

    @property
    def susp_baja_impago_en_curso(self):
        data = ''
        try:
            data = self.datos_solicitud.SuspBajaImpagoEnCurso.text
        except AttributeError:
            pass
        return data


class DatosActivacion(object):

    def __init__(self, data):
        self.datos_activacion = data

    @property
    def fecha_activacion(self):
        data = ''
        try:
            data = self.datos_activacion.FechaActivacion.text
        except AttributeError:
            pass
        return data

    @property
    def en_servicio(self):
        data = ''
        try:
            data = self.datos_activacion.EnServicio.text
        except AttributeError:
            pass
        return data

    @property
    def ind_esencial(self):
        data = ''
        try:
            data = self.datos_activacion.IndEsencial.text
        except AttributeError:
            pass
        return data

    @property
    def fecha_ultimo_movimiento_ind_esencial(self):
        data = False
        try:
            data = self.datos_activacion.FechaUltimoMovimientoIndEsencial.text
        except AttributeError:
            pass
        return data


class DatosNotificacion(object):

    def __init__(self, data):
        self.datos_notificacion = data

    @property
    def fecha_activacion(self):
        data = ''
        try:
            data = self.datos_notificacion.FechaActivacion.text
        except AttributeError:
            pass
        return data



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from libcomxml.core import XmlModel, XmlField
from gestionatr.output.messages.base_gas import Heading
from gestionatr.output.messages.sw_a1_05 import Registerdoclist, Registerdoc


# Paso 01
class MensajeA146(XmlModel):
    _sort_order = ('mensaje', 'heading', 'a146')

    def __init__(self):
        self.doc_root = None
        self.mensaje = XmlField(
            'sctdapplication', attributes={'xmlns': 'http://localhost/sctd/A146'}
        )
        self.heading = Heading()
        self.a146 = A146()
        super(MensajeA146, self).__init__('sctdapplication', 'mensaje')


class A146(XmlModel):

    _sort_order = ('a146', 'comreferencenum', 'comreferencenumdes', 'reqdate', 'reqhour', 'annulmentreason',
                   'operationtype', 'cups', 'claimtype', 'claimsubtype', 'extrainfo', 'registerdoclist')

    def __init__(self):
        self.a146 = XmlField('a146')
        self.comreferencenum = XmlField('comreferencenum')
        self.comreferencenumdes = XmlField('comreferencenumdes')
        self.reqdate = XmlField('reqdate')
        self.reqhour = XmlField('reqhour')
        self.annulmentreason = XmlField('annulmentreason')
        self.operationtype = XmlField('operationtype')
        self.cups = XmlField('cups')
        self.claimtype = XmlField('claimtype')
        self.claimsubtype = XmlField('claimsubtype')
        self.extrainfo = XmlField('extrainfo')
        self.registerdoclist = []
        super(A146, self).__init__('a146', 'a146')




# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from .message import Message, except_f1
from datetime import datetime
from gestionatr.utils import get_rec_attr
from .Deadlines import ProcessDeadline
from .F1 import EnergiaActiva, Autoconsumo, InformacionAlConsumidor, Expediente

MAGNITUDS_OCSUM = {
    'AE': 'A',
    'AS': 'S',
    'R1': 'R',
    'R4': 'RC',
    'PM': 'M',
    'EP': 'EP'
}

# Retorna el nom de periode segons el periode d'OCSUM
PERIODE_OCSUM = {
    '01': 'P1',  # Punta + Llano
    '03': 'P2',  # Valle
    '10': 'P1',  # Totalizador (2.0A y 2.1A)
    '20': 'P1',  # Totalizador (2.1DHA y 2.0DHA)
    '21': 'P1',  # P1 Tarifes: 004, 006
    '22': 'P2',  # P2 Tarifes: 004, 006
    '31': 'P1',  # P1 Tarifa 003
    '32': 'P2',  # P2 Tarifa 003
    '33': 'P3',  # P3 Tarifa 003
    '41': 'P1',  # P1 Tarifa 011
    '42': 'P2',  # P2 Tarifa 011
    '43': 'P3',  # P3 Tarifa 011
    '61': 'P1',  # Periodo 1 Tarifes: 003, 011, 012 - 017
    '62': 'P2',  # Periodo 2 Tarifes: 003, 011, 012 - 017
    '63': 'P3',  # Periodo 3 Tarifes: 003, 011, 012 - 017
    '64': 'P4',  # Periodo 4 Tarifes: 003, 011, 012 - 017
    '65': 'P5',  # Periodo 5 Tarifes: 003, 011, 012 - 017
    '66': 'P6',  # Periodo 6 Tarifes: 003, 011, 012 - 017
    '80': 'P1',  # Totalizador Maximetro Tarifa 007
    '81': 'P1',  # P1 Tarifa 007
    '82': 'P2',  # P2 Tarifa 007
    '83': 'P3',  # P3 Tarifa 007
    '90': 'P1',  # P1 de peaje de acceso 2.0TD.
    '91': 'P1',  # P1 de peaje de acceso 2.0TD.
    '92': 'P2',  # P2 de peaje de acceso 2.0TD.
    '93': 'P3',  # P3 de peaje de acceso 2.0TD.
    'A0': 'P1',  # P1 de los peajes de acceso 3.0TD, 3.0TDVE, 6.1TD, 6.1TDVE, 6.2TD. 6.3TD. 6.4TD
    'A1': 'P1',  # P1 de los peajes de acceso 3.0TD, 3.0TDVE, 6.1TD, 6.1TDVE, 6.2TD. 6.3TD. 6.4TD
    'A2': 'P2',  # P2 de los peajes de acceso 3.0TD, 3.0TDVE, 6.1TD, 6.1TDVE, 6.2TD. 6.3TD. 6.4TD
    'A3': 'P3',  # P3 de los peajes de acceso 3.0TD, 3.0TDVE, 6.1TD, 6.1TDVE, 6.2TD. 6.3TD. 6.4TD
    'A4': 'P4',  # P4 de los peajes de acceso 3.0TD, 3.0TDVE, 6.1TD, 6.1TDVE, 6.2TD. 6.3TD. 6.4TD
    'A5': 'P5',  # P5 de los peajes de acceso 3.0TD, 3.0TDVE, 6.1TD, 6.1TDVE, 6.2TD. 6.3TD. 6.4TD
    'A6': 'P6',  # P6 de los peajes de acceso 3.0TD, 3.0TDVE, 6.1TD, 6.1TDVE, 6.2TD. 6.3TD. 6.4TD

}

SKIP_TOTALITZADORS = ('00', '60')

class Q1(Message, ProcessDeadline):
    """Clase que implementa Q1."""

    @property
    def get_cabecera_codigo_fiscal_factura(self):
        val = self.head.CodigoFiscalFactura.text
        if not val:
            raise except_f1('Error', u'Q1 sin CodigoFiscalFactura')
        return val

    @property
    def get_cabecera_tipo_factura(self):
        val = self.head.TipoFactura.text
        if not val:
            raise except_f1('Error', u'Q1 sin TipoFactura')
        return val

    @property
    def get_cabecera_motivo_facturacion(self):
        val = self.head.MotivoFacturacion.text
        if not val:
            raise except_f1('Error', u'Q1 sin MotivoFacturacion')
        return val

    @property
    def get_cabecera_codigo_factura_rectificada_anulada(self):
        val = self.head.CodigoFacturaRectificadaAnulada.text
        if not val:
            return False
        return val

    @property
    def get_cabecera_expediente(self):
        if hasattr(self.head, 'Expediente'):
            return Expediente(self.head.Expediente)
        return None

    @property
    def datos(self):
        if hasattr(self.obj, 'Datos'):
            return Datos(self.obj.Datos)
        return None

    @property
    def energia_activa(self):
        if hasattr(self.obj, 'EnergiaActiva'):
            return EnergiaActiva(self.obj.EnergiaActiva)
        return None

    @property
    def autoconsumo(self):
        if hasattr(self.obj, 'Autoconsumo'):
            return Autoconsumo(self.obj.Autoconsumo)
        return None

    @property
    def medidas(self):
        medidas = []
        data = get_rec_attr(self.obj, 'Medidas')
        for medida in data:
            medidas.append(Medida(medida))

        return medidas

    @property
    def comptadors(self):
        """Retorna totes les lectures en una llista de comptadors"""
        contadores_agrupados = {}
        for medida in self.medidas:
            for aparato in medida.aparatos:
                contadores_agrupados.setdefault(
                    aparato.numero_serie, []
                ).append(aparato)

        contadores = []
        for lista_aparatos in contadores_agrupados.values():
            fd = aparato.fecha_desde
            fh = aparato.fecha_hasta
            contadores.append((fd, fh, aparato))
        return [a[2] for a in sorted(contadores, key=lambda x: x[0])]

    @property
    def informacion_al_consumidor(self):
        if hasattr(self.obj, 'InformacionAlConsumidor'):
            return InformacionAlConsumidor(self.obj.InformacionAlConsumidor)
        return None


class Datos(object):

    def __init__(self, data):
        self.datos = data

    @property
    def tipo_autoconsumo(self):
        data = ''
        try:
            data = self.datos.TipoAutoconsumo.text
        except AttributeError:
            pass
        return data

    @property
    def tipo_subseccion(self):
        data = ''
        try:
            data = self.datos.TipoSubseccion.text
        except AttributeError:
            pass
        return data

    @property
    def tipo_cups(self):
        data = ''
        try:
            data = self.datos.TipoCUPS.text
        except AttributeError:
            pass
        return data

    @property
    def marca_medida_con_perdidas(self):
        data = ''
        try:
            data = self.datos.MarcaMedidaConPerdidas.text
        except AttributeError:
            pass
        return data

    @property
    def vas_trafo(self):
        data = ''
        try:
            data = self.datos.VAsTrafo.text
        except AttributeError:
            pass
        return data

    @property
    def porcentaje_perdidas(self):
        data = ''
        try:
            data = self.datos.PorcentajePerdidas.text
        except AttributeError:
            pass
        return data

    @property
    def indicativo_curva_carga(self):
        data = ''
        try:
            data = self.datos.IndicativoCurvaCarga.text
        except AttributeError:
            pass
        return data

    @property
    def _periodo_cch(self):
        if hasattr(self.datos, 'PeriodoCCH'):
            return self.datos.PeriodoCCH
        return None

    @property
    def fecha_desde_cch(self):
        if hasattr(self._periodo_cch, 'FechaDesdeCCH'):
            return self._periodo_cch.FechaDesdeCCH.text.strip()
        return None

    @property
    def fecha_hasta_cch(self):
        if hasattr(self._periodo_cch, 'FechaHastaCCH'):
            return self._periodo_cch.FechaHastaCCH.text.strip()
        return None

    @property
    def _periodo(self):
        if hasattr(self.datos, 'Periodo'):
            return self.datos.Periodo
        return None

    @property
    def fecha_desde_factura(self):
        if hasattr(self._periodo, 'FechaDesdeFactura'):
            return self._periodo.FechaDesdeFactura.text.strip()
        return None

    @property
    def fecha_hasta_factura(self):
        if hasattr(self._periodo, 'FechaHastaFactura'):
            return self._periodo.FechaHastaFactura.text.strip()
        return None

    @property
    def numero_dias(self):
        if hasattr(self._periodo, 'NumeroDias'):
            return int(self._periodo.NumeroDias.text.strip())
        return None

    @property
    def tipo_pm(self):
        if hasattr(self.datos, 'TipoPM'):
            return self.datos.TipoPM.text.strip()
        return None


class Medida(object):
    def __init__(self, data):
        self.datos_medida = data

    @property
    def cod_pm(self):
        cod_pm = self.datos_medida.CodPM
        return cod_pm

    @property
    def aparatos(self):
        aparatos = []
        data = get_rec_attr(self.datos_medida, 'ModeloAparato')
        for aparato in data:
            aparatos.append(ModeloAparato(aparato))
        return aparatos

class ModeloAparato(object):

    def __init__(self, data):
        self.modelo_aparato = data

    @property
    def tipo_aparato(self):
        data = ''
        try:
            data = self.modelo_aparato.TipoAparato.text
        except AttributeError:
            pass
        return data

    @property
    def marca_aparato(self):
        data = ''
        try:
            data = self.modelo_aparato.MarcaAparato.text
        except AttributeError:
            pass
        return data

    @property
    def numero_serie(self):
        data = ''
        try:
            data = self.modelo_aparato.NumeroSerie.text
        except AttributeError:
            pass
        return data

    @property
    def tipo_dhedm(self):
        data = ''
        try:
            data = self.modelo_aparato.TipoDHEdM.text
        except AttributeError:
            pass
        return data

    @property
    def integradores(self):
        integradores = []
        data = get_rec_attr(self.modelo_aparato, 'Integrador')
        for i in data:
            integradores.append(Integrador(i))
        return integradores

    @property
    def fecha_desde(self):
        fecha_desde = False
        for integrador in self.integradores:
            fecha_in_compt = datetime.strptime(
                integrador.lectura_desde.fecha[:10], '%Y-%m-%d'
            )
            if not fecha_desde or fecha_in_compt < fecha_desde:
                fecha_desde = fecha_in_compt

        return fecha_desde

    @property
    def fecha_hasta(self):
        fecha_hasta = False
        for integrador in self.integradores:
            fecha_fi_compt = datetime.strptime(
                integrador.lectura_hasta.fecha[:10], '%Y-%m-%d'
            )
            if not fecha_hasta or fecha_fi_compt > fecha_hasta:
                fecha_hasta = fecha_fi_compt

        return fecha_hasta

    @property
    def lectures(self,  tipos=None):
        """Retorna totes les lectures en una llista de Lectura"""
        lectures = []
        try:
            for integrador in self.integradores:
                if not tipos or (integrador.tipus and integrador.tipus in tipos):
                    integrador.comptador = self
                    lectures.append(integrador)
        except AttributeError:
            pass
        lectures = sorted(lectures, key=lambda x: x.lectura_desde.fecha)
        return lectures

    @property
    def gir_contador(self):
        giros_lect = [
            int.gir_contador for int in self.integradores
        ]

        return max(giros_lect)

class Integrador(object):

    def __init__(self, data):
        self.integrador = data
        self._lectura_desde = None
        self._lectura_hasta = None
        self._magnitud = None
        self._periode = None
        self._numero_ruedas_enteras = None
        self._ajuste = None
        self.comptador = None

    @property
    def tipus(self):
        return MAGNITUDS_OCSUM.get(self.magnitud)

    @property
    def ometre(self):
        return self.codigo_periodo in SKIP_TOTALITZADORS

    @property
    def magnitud(self):
        data = ''
        try:
            data = self.integrador.Magnitud.text
        except AttributeError:
            pass
        return data

    @property
    def periode(self):
        p = PERIODE_OCSUM.get(self.codigo_periodo, None)
        if not p and self.codigo_periodo in PERIODE_OCSUM.values():
            return self.codigo_periodo
        if self.codigo_periodo == '93' and p == 'P3' and self.magnitud in ('PM', 'EP'):
            return "P2"
        return p

    @property
    def codigo_periodo(self):
        data = ''
        try:
            data = self.integrador.CodigoPeriodo.text
        except AttributeError:
            pass
        return data

    @property
    def constante_multiplicadora(self):
        data = ''
        try:
            data = self.integrador.ConstanteMultiplicadora.text
        except AttributeError:
            pass
        return data

    @property
    def numero_ruedas_enteras(self):
        data = False
        try:
            data = int(self.integrador.NumeroRuedasEnteras.text)
        except AttributeError:
            pass
        return data

    @property
    def numero_ruedas_decimales(self):
        data = False
        try:
            data = int(self.integrador.NumeroRuedasDecimales.text)
        except AttributeError:
            pass
        return data

    @property
    def gir_contador(self):
        ruedas_enteras = self.numero_ruedas_enteras
        if ruedas_enteras == 99:
            return 10
        return (10 ** ruedas_enteras)

    @property
    def consumo_calculado(self):
        data = False
        try:
            data = float(self.integrador.ConsumoCalculado.text.strip())
        except AttributeError:
            pass
        return data

    @property
    def lectura_desde(self):
        data = ''
        try:
            data = LecturaInfo(self.integrador.LecturaDesde)
        except AttributeError:
            pass
        return data

    @property
    def fecha_lectura_desde(self):
        return self.lectura_desde.fecha

    def valor_lectura_desde(self):
        return self.lectura_desde.lectura

    @property
    def lectura_hasta(self):
        data = ''
        try:
            data = LecturaInfo(self.integrador.LecturaHasta)
        except AttributeError:
            pass
        return data

    @property
    def fecha_lectura_hasta(self):
        return self.lectura_hasta.fecha

    @property
    def valor_lectura_hasta(self):
        return self.lectura_hasta.lectura


    @property
    def ajuste(self):
        data = ''
        try:
            data = Ajuste(self.integrador.Ajuste)
        except AttributeError:
            pass
        return data

    @property
    def anomalia(self):
        data = ''
        try:
            data = Anomalia(self.integrador.Anomalia)
        except AttributeError:
            pass
        return data

    @property
    def fecha_hora_maximetro(self):
        data = False
        try:
            data = self.integrador.FechaHoraMaximetro.text
        except AttributeError:
            pass
        return data


class LecturaInfo(object):

    def __init__(self, data):
        self.lect = data

    @property
    def fecha(self):
        data = False
        try:
            data = self.lect.Fecha.text
        except AttributeError:
            pass
        return data

    @property
    def procedencia(self):
        data = ''
        try:
            data = self.lect.Procedencia.text
        except AttributeError:
            pass
        return data

    @property
    def lectura(self):
        data = False
        try:
            data = float(self.lect.Lectura.text.strip())
        except AttributeError:
            pass
        return data


class Ajuste(object):

    def __init__(self, data):
        self.ajuste = data

    @property
    def codigo_motivo_ajuste(self):
        data = ''
        try:
            data = self.ajuste.CodigoMotivoAjuste.text
        except AttributeError:
            pass
        return data

    @property
    def ajuste_por_integrador(self):
        data = ''
        try:
            data = self.ajuste.AjustePorIntegrador.text
        except AttributeError:
            pass
        return data

    @property
    def comentarios(self):
        data = ''
        try:
            data = self.ajuste.Comentarios.text
        except AttributeError:
            pass
        return data


class Anomalia(object):

    def __init__(self, data):
        self.anomalia = data

    @property
    def tipo_anomalia(self):
        data = ''
        try:
            data = self.anomalia.TipoAnomalia.text
        except AttributeError:
            pass
        return data

    @property
    def comentarios(self):
        data = ''
        try:
            data = self.anomalia.Comentarios.text
        except AttributeError:
            pass
        return data

